import unittest
import re
import os
import importlib
import sys
import signal
import json

def discover(start_dir='.', pattern='', top_level_dir=None):

    if top_level_dir == None:
        top_level_dir = start_dir

    if os.sep in pattern and pattern[-3:] == '.py':
        # This approach was chosen over the more explicit use of paths with imp/importlib
        #   since they appear to be bugged and pick up parent modules with the same name as well.
        sys.path.insert(0, start_dir)
        name = pattern[:-3].replace(os.path.sep, '.')
        module = importlib.import_module(name)
        del sys.path[0]

        return unittest.defaultTestLoader.loadTestsFromModule(module)

    kwargs = {
        'start_dir': start_dir,
        'top_level_dir': top_level_dir,
    }
    search = False

    if os.sep in pattern:
        kwargs['start_dir'] += os.path.sep + pattern
    elif pattern[-3:] == '.py':
        kwargs['pattern'] = pattern
    elif pattern != '':
        search = True

    suite = unittest.defaultTestLoader.discover(**kwargs)

    if search:
        tests = list_tests(suite)
        suite = unittest.TestSuite()

        for test in tests:
            if re.search(pattern, test.id()) is not None:
                suite.addTest(test)

    return suite

def list_tests(suite):
    tests = []
    items = [item for item in suite]

    while len(items):
        next_items = []
        for item in items:
            if hasattr(item, '__iter__'):
                next_items += [ next_item for next_item in item ]
            else:
                tests.append(item)
        items = next_items

    return tests

def run(**kwargs):

    if sys.path[0] != os.getcwd():
        sys.path.insert(0, os.getcwd())

    if kwargs['verbose']:
        verbosity = 2
    elif kwargs['quiet']:
        verbosity = 0
    else:
        verbosity = 1

    last = kwargs['runner'].rfind('.')
    package = kwargs['runner'][:last]
    runner = kwargs['runner'][last + 1:]
    try:
        module = importlib.import_module(package)
    except ImportError:
        if package[:6] == 'sputr.':
            package = package[6:]
            module = importlib.import_module(package)
    runner = getattr(module, runner)(
        verbosity=verbosity,
        failfast=kwargs['failfast'],
        buffer=kwargs['buffer']
    )
    runner.color = kwargs['color']

    suite = discover(
        start_dir=kwargs['start_dir'], 
        pattern=kwargs['pattern'], 
        top_level_dir=kwargs['top_level_dir']
    )

    if kwargs['catch']:
        unittest.installHandler()
    result = runner.run(suite)
    if kwargs['catch']:
        unittest.removeHandler()

    sys.exit(0 if result.wasSuccessful() and (result.testsRun > 0 or kwargs['allow_none']) else 1)




from __future__ import absolute_import
import sys
import os
import json

import sputr

def cli():
    if len(sys.argv) >= 3 and sys.argv[1] == '--json':
        sputr.run(**json.loads(sys.argv[2][1:-1]))

    if 'SPUTR_OPTIONS' in os.environ:
        sys.argv += os.environ['SPUTR_OPTIONS'].split()
    
    from .click_cli import click_cli
    click_cli()


if __name__ == '__main__':
    cli()




from __future__ import absolute_import
import click

import sputr
from .config import version

@click.command(
    help='Simple Python Unit Test Runner',
    epilog="""\b\bExamples:\n
sputr                      -run all tests in the current directory\n
sputr dirname              -run all tests in the specified directory\n
sputr filename.py          -run all tests in files with this name\n
sputr dirname/filename.py  -run all tests in a specific file\n
sputr test_name            -run all tests that match a pattern\n
sputr test_name_*          -run all tests that match a pattern
    """,
    context_settings={
        'help_option_names': ['-h','--help'],
    }
)
@click.argument('pattern', default='')
@click.version_option(version=version)
@click.option('--start_dir', '-s', default='.', show_default=True, help='Directory to start discovery')
@click.option('--verbose', '-v', is_flag=True, help='More output')
@click.option('--quiet', '-q', is_flag=True, help='Less output')
@click.option('--failfast', '-f', is_flag=True, help='Stop execution on test failure or error')
@click.option('--buffer', '-b', is_flag=True, help='Buffer stdout and stderr during tests')
@click.option('--catch', '-c', is_flag=True, help='Display test results after keyboard interrupt (control-c)')
@click.option('--top_level_dir', '-t', help='Top level directory of the project')
@click.option(
    '--runner', '-r', 
    default='unittest.TextTestRunner', 
    show_default=True, 
    help='Test runner to use when running tests'
)
@click.option('--color', is_flag=True, help='Add colors to output if supported by the test runner')
@click.option('--json', help='Include options as JSON instead of requiring click')
@click.option('--allow-none', is_flag=True, help='Succeed when zero tests were run')
def click_cli(*args, **kwargs):
    sputr.run(*args, **kwargs)




import unittest
import subprocess
import sputr

class SimpleTestResult(unittest.TestResult):

    def stopTest(self, test):
        self.log_status(self.testsRun, len(self.failures), len(self.errors), len(self.skipped))

class SimpleTestRunner():

    def color(value):
        return '\x1b[%sm' % value
    colors = {
        'red': color('0;31;40'),
        'green': color('0;32;40'),
        'yellow': color('0;33;40'),
        'purple': color('0;35;50'),
        'cyan': color('0;36;40'),
        'end': color('0'),
    }

    status_messages = {
        0: '{purple}%s {green}%s {red}%s {yellow}%s {cyan}%s{end}',
        1: '{purple}%s total, {green}%s ran, {red}%s failed, {yellow}%s error, {cyan}%s skipped{end}',
        2: '{purple}%s will be run.\n{green}%s have been run.\n{red}%s have failed.\n{yellow}%s have returned an error.\n{cyan}%s have been skipped.{end}',
    }
    result_messages = {
        'failures': '{red}%s\n%s{end}',
        'errors': '{yellow}%s\n%s{end}',
    }

    def __init__(self, verbosity=1, failfast=False, buffer=False):
        self.verbosity = verbosity
        self.failfast = failfast
        self.buffer = buffer

    def log(self, message=''):
        print(message)

    def num_status_lines(self):
        return 5 if self.verbosity == 2 else 1

    def log_status(self, *args):
        # Move cursor to the start of the previous line
        subprocess.call(['tput', 'cuu', str(self.num_status_lines()), '&&', 'tput', 'el'])
        self.log(self.status_messages[self.verbosity] % ((self.test_count,) + args))

    def run(self, suite):
        if not self.color:
            for name, value in self.colors.items():
                self.colors[name] = ''
        for level, message in self.status_messages.items():
            self.status_messages[level] = message.format(**self.colors)
        for level, message in self.result_messages.items():
            self.result_messages[level] = message.format(**self.colors)

        if hasattr(suite, 'countTestCases'):
            self.test_count = suite.countTestCases()
        else:
            self.test_count = 1
        result = SimpleTestResult()
        result.failfast = self.failfast
        result.buffer = self.buffer
        result.log_status = self.log_status

        self.log('\n' * (self.num_status_lines() - 1))
        suite.run(result)

        if self.verbosity > 0:
            for type in ['failures', 'errors']:
                if len(getattr(result, type)) > 0:
                    self.log(type.upper() + '\n')
                    for incident in getattr(result, type):
                        self.log(self.result_messages[type] % (incident[0].id(), incident[1]))

        return result

